use leptos::*;
use strsim::normalized_damerau_levenshtein;

use crate::util::surrealdb::Participant;

fn sort_participants(participants: Vec<Participant>, search: String) -> Vec<Participant> {
    let mut filtered_sorted_list: Vec<(Participant, f64)> = participants
        .into_iter()
        .map(|participant| {
            (
                participant.clone(),
                normalized_damerau_levenshtein(
                    &participant.name.to_lowercase(),
                    &search.to_lowercase(),
                ),
            )
        })
        .collect();

    filtered_sorted_list.sort_by(|a, b| {
        let (_, sim_score_a) = a;
        let (_, sim_score_b) = b;
        sim_score_b
            .partial_cmp(sim_score_a)
            .unwrap_or(std::cmp::Ordering::Equal)
    });

    filtered_sorted_list
        .into_iter()
        .map(|(item, _)| item)
        .collect()
}

/// Navigation bar
#[component]
pub fn Add() -> impl IntoView {
    let websocket = expect_context::<crate::util::surrealdb::SurrealContext>();
    let participants = expect_context::<crate::util::surrealdb::ParticipantsContext>();

    let (name, set_name) = create_signal(String::from(""));
    let (error, set_error) = create_signal(String::from(""));

    let participants_sorted =
        create_memo(move |_| sort_participants(participants.read.get(), name.get()));

    let on_submit = move |ev: leptos::ev::SubmitEvent| {
        ev.prevent_default();
        set_error.set(String::from(""));
        /*
        match websocket.add_person(name.get(), group.get()) {
            Ok(_) => set_name.set(String::from("")),
            Err(err) => set_error.set(err),
        }
        */
    };

    view! {
        <h1>"Tijd toevoegen"</h1>
        <form class="add" on:submit=on_submit>
            <input type="text"
                placeholder="Name"
                on:input=move |ev| {
                    set_name.set(event_target_value(&ev));
                }
                prop:value=name
            />
            <ul class="participants-container">
                {move || participants_sorted.get().into_iter().map(|participant| view! {
                    <li>{participant.name + " " + "(" + &participant.group[6..].to_string() + ")" }</li>
                }).collect_view()}
            </ul>
            <input type="submit" value="Submit" />
        </form>
        <p class="error">
            { error }
        </p>
    }
}
